// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "ui/UIAlignableElement.hpp"

namespace Dawn {
  class UIContainer : public UIAlignableElement {
    private:
      std::vector<std::shared_ptr<UIElement>> children;

    public:
      std::vector<std::shared_ptr<UIElement>> getChildren() override;

      float_t getContentWidth() override;
      float_t getContentHeight() override;
      
      /**
       * Appends a child to this container.
       * 
       * @param child Child to append.
       */
      void appendChild(std::shared_ptr<UIElement> child);

      /**
       * Removes a child from this container.
       * 
       * @param child Child to remove.
       */
      void removeChild(std::shared_ptr<UIElement> child);

      /**
       * Removes all children from this container.
       */
      void clearChildren();
  };
}