// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "InputManager.hpp"
#include "game/Game.hpp"

using namespace Dawn;

void InputManager::init(const std::shared_ptr<Game> game) {
  auto window = game->renderHost.window;
  
  glfwSetCursorPosCallback(window, [](
    GLFWwindow* window,
    double_t x,
    double_t y
  ) {
    auto game = (Game*)glfwGetWindowUserPointer(window);
    game->inputManager.rawInputValues[INPUT_MANAGER_AXIS_MOUSE_X] = (float_t) x;
    game->inputManager.rawInputValues[INPUT_MANAGER_AXIS_MOUSE_Y] = (float_t) y;
  });

  glfwSetMouseButtonCallback(window, [](
    GLFWwindow* window,
    int32_t button,
    int32_t action,
    int32_t mods
  ) {
    auto game = (Game*)glfwGetWindowUserPointer(window);
    game->inputManager.rawInputValues[INPUT_MANAGER_AXIS_MOUSE_0 + button] = (
      action == GLFW_PRESS ? 1.0f : 0.0f
    );
  });

  glfwSetMouseButtonCallback(window, [](
    GLFWwindow* window,
    int32_t button,
    int32_t action,
    int32_t mods
  ) {
    auto game = (Game*)glfwGetWindowUserPointer(window);
    game->inputManager.rawInputValues[button] = (
      action == GLFW_PRESS ? 1.0f : 0.0f
    );
  });
}

float_t InputManager::getInputValue(int32_t axis) {
  auto exist = rawInputValues.find(axis);
  if(exist == rawInputValues.end()) return 0.0f;
  return exist->second;
}

InputManager::~InputManager() {
  // Nothing to do here.
}