
// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "scenes/SceneList.hpp"
#include "component/display/Camera.hpp"
#include "prefab/SimpleSpinningCube.hpp"
#include "component/display/material/SimpleTexturedMaterial.hpp"
#include "display/mesh/CubeMesh.hpp"

using namespace Dawn;

void Dawn::testScene(Scene &s) {
  auto cameraItem = s.createSceneItem();
  auto camera = cameraItem->addComponent<Camera>();
  cameraItem->lookAt({ 3, 3, 3 }, { 0, 0, 0 }, { 0, 1, 0 });
  camera->clipFar = 99999.99f;

  auto cube = s.createSceneItem();
  auto cubeMesh = std::make_shared<Mesh>();
  auto cubeRenderer = cube->addComponent<MeshRenderer>();
  auto cubeMaterial = cube->addComponent<SimpleTexturedMaterial>();
  
  cubeRenderer->mesh = cubeMesh;
  cubeMesh->createBuffers(CUBE_VERTICE_COUNT, CUBE_INDICE_COUNT);
  CubeMesh::buffer(cubeMesh, glm::vec3(0,0,0), glm::vec3(1, 1, 1), 0, 0);
}